{
 copyright   : daniel lorch 1999 <daniel@lorch.cc>
 name        : raylight URL label
 version     : 1.00 (initial release)
 language    : everything greater than delphi 2.0, should theoretically
               also compile on 16-bit versions.
 description : allows you to have 'hyperlinks' in your applications,
               which will start an url or e-mail window. it also
               has the well known 'OnMouseOver' effect implemented
               and lets you choose between 3 cursors.
 legal       : this src is PD, you may use it for whatever you want,
               credits would be nice. I take no responsibility for any
               damage or consequences directly or indirectly, intentionally or
               unintentionally, which may arise from using this program.
               YOU USE IT AT YOUR OWN RISK!


 modifying   : if you've made any improvements, please send me a copy of it.
 usage       : raylight URL label inherits from TLabel. new properties are:
               -ColorActive    : color of label when 'clicked'
               -ColorMouseOver : color of label when mouse is moving over it.
               -CursorMouseOver: you can choose between 3 cursors. this one
                                 is a bit complicated, but I didn't find a
                                 proper way to solve the problem. if you want to
                                 have a 'normal cursor' instead of a 'browser cursor'
                                 you first have to set 'CursorMouseOver' to 'crDefaultCursor'
                                 and then you can choose a cursor in 'Cursor'. you don't
                                 have to do this, when you want to have a 'browser cursor'
                                 instead of a 'normal cursor'. choosing a 'browser cursor'
                                 updates automatically the 'Cursor' property.
               -PopupMenu      : let's you choose a popup-menu when label is clicked with
                                 the right mouse-button. there is already a 'built-in'
                                 menu which allows the user to copy the link
                                 location (property 'url')
               -PopupString    : the text displayed in the popup-menu above.
               -Url            : the most important property. this is the url or e-mail
                                 executed. please read html-tutorials for more
                                 informations about this. examples:
                                 - http://raylight.hypermart.net
                                 - mailto:daniel@lorch.cc
                                 - mailto:daniel@lorch.cc?subject=cool component
}

unit raylightURLLabel;

interface

uses
  Windows, Messages, SysUtils, Controls, Forms, Graphics, Dialogs,
  StdCtrls, Classes, registry, shellapi, Clipbrd, Menus;

const
  crBrowserHand = 5;

type TCustomCursor=(crDefaultCursor,crMozilla,crMSIE,crDelphiHelp);

type
  TRaylightURLLabel = class(TLabel)
  private
    FColorNormal : TColor;
    FColorMouseOver : TColor;
    FColorActive : TColor;

    FMouseDown : boolean;
    FMouseIn : boolean;

    Furl : String;

    FCustomCursor : TCustomCursor;

    Menu: TPopupMenu;
    MenuItem: TMenuItem;
    FPopupString : String;

    procedure OnMenuClick(Sender: TObject);
    procedure OnMenuPopup(Sender: TObject);

    procedure CMMouseEnter(var Message: TMessage); message CM_MOUSEENTER;
    procedure CMMouseLeave(var Message: TMessage); message CM_MOUSELEAVE;
    procedure MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;
    procedure MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer); override;

    procedure adjustLabel;
    function findBrowser : string;
    function replaceString(param, what2replace, replaceBy : string) : string;
    procedure SetCustomCursor(value : TCustomCursor);
    procedure SetPopupString(value : String);
  protected
    { Protected-Deklarationen }
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property ColorMouseOver : TColor read FColorMouseOver write FColorMouseOver;
    property ColorActive : TColor read FColorActive write FColorActive;

    property OnMouseDown;
    property OnMouseMove;
    property OnMouseUp;
    property OnClick;

    property Url : string read Furl write Furl;

    property CursorMouseOver : TCustomCursor read FCustomCursor write SetCustomCursor;

    property PopupString : String read FPopupString write SetPopupString;
  end;

const
 RaylightUrl = 'http://raylight.hypermart.net';

procedure Register;

implementation

{$R browserCursors.dcr}

constructor TRaylightURLLabel.create(AOwner: TComponent);
const
 crMyCursor = 5;
begin
 inherited Create(AOwner);

 Font.Color := clBlue;
 Font.Style := [fsUnderline];

 FColorNormal := Font.Color;
 FColorMouseOver := clRed;
 FColorActive := clPurple;

 FPopupString := 'Copy Link Location';

 FUrl := RaylightUrl;

 Menu := TPopupMenu.Create(Self);
 Menu.OnPopup := OnMenuPopup;
 MenuItem := TMenuItem.Create(Menu);
 with MenuItem do begin
  Caption := FPopupString;
  Name := 'MenuItem';
  OnClick := OnMenuClick;
 end;
 Menu.Items.Add(MenuItem);
 PopupMenu := Menu;

 CursorMouseOver := crMozilla;
 Screen.Cursors[crBrowserHand] := LoadCursor(HInstance, PChar('MOZILLA'));
 Cursor := crBrowserHand;
end;

destructor TRaylightURLLabel.destroy;
begin
 Menu.Free;
 inherited destroy;
end;

procedure TRaylightURLLabel.SetCustomCursor(value: TCustomCursor);
begin
 case value of
  crDefaultCursor : Cursor := crDefault;
  crMozilla       : Screen.Cursors[crBrowserHand] := LoadCursor(HInstance, PChar('MOZILLA'));
  crMSIE          : Screen.Cursors[crBrowserHand] := LoadCursor(HInstance, PChar('MSIE'));
  crDelphiHelp    : Screen.Cursors[crBrowserHand] := LoadCursor(HInstance, PChar('DELPHIHAND'));
 end;

 if value <> crDefaultCursor then Cursor := crBrowserHand;

 FCustomCursor := value;
end;

procedure TRaylightURLLabel.SetPopupString(value : String);
begin
 Menu.items[0].caption := value;
 FPopupString := value;
end;

function TRaylightURLLabel.findBrowser : string;
var
 fileType : string;
 browserPath : string;
 i : integer;
const
 extensionCount = 4;
 htmlextensions : array[0..extensionCount-1] of string = ('.html','.htm','.shtml','.shtm');
begin

 with TRegistry.Create do
 try
  RootKey := HKEY_CLASSES_ROOT;

  browserPath:='';
  i:=0;

  while (length(browserPath) = 0)  and (i <= extensionCount) do
  begin

   if OpenKey ('\' + htmlextensions[i], false) = true then
   begin
    fileType := ReadString('');

    if length(fileType) <> 0 then
     if OpenKey ('\' + fileType + '\shell\open\command', false) = true then
      browserPath := ReadString('');
   end;

   inc(i);

  end; //end while


  CloseKey;
 finally
  Free;
 end;

 result := browserpath;
end;

//a hack, not very good programmed. if you've got a better function, mail me.
function TRaylightURLLabel.replaceString(param, what2replace, replaceBy : string) : string;
var x   : integer;
    s1   : string;
    s2   : string;
begin
 while pos(what2replace,param) <> 0 do
 begin
  s1 := copy(param,1,pos(what2replace,param)-1);
  s2 := copy(param,pos(what2replace,param)+length(what2replace),length(param));
  param := s1+replaceBy+s2;
 end;

 result := param;
end;

procedure TRaylightURLLabel.CMMouseEnter(var Message: TMessage);
begin
 FMouseIn := true;

 adjustLabel;
end;

procedure TRaylightURLLabel.CMMouseLeave(var Message: TMessage);
begin
 FMouseIn := false;

 adjustLabel;
end;

procedure TRaylightURLLabel.MouseDown(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
begin
 if Button = mbLeft then FMouseDown := true;
 FMouseIn := true;

 adjustLabel;
 Inherited MouseDown(Button, Shift, X, Y);
end;

procedure TRaylightURLLabel.MouseUp(Button: TMouseButton; Shift: TShiftState; X, Y: Integer);
var
 path : string;
 params : string;
begin
 if Button = mbLeft then FMouseDown := false;
 adjustLabel;

 if FMouseIn and (Button = mbLeft) then
 begin
  path := findbrowser;
  while pos('%1', path) <> 0 do path := replaceString(path, '%1', '');

  params := Furl;

  if fileexists(path)
   //start browser associated to html files
   then shellexecute(GetDesktopWindow(),nil,pchar(path),pchar(params),nil,SW_SHOW)
   //let windows do the job, usually starts IE
   else shellexecute(GetDesktopWindow(),nil,pchar(params),nil,nil,SW_SHOW);
 end;
 
 Inherited MouseUp(Button, Shift, X, Y);
end;

procedure TRaylightURLLabel.adjustLabel;
begin
 if FMouseIn then
 begin

  if FMouseDown
   then Font.Color := FColorActive
   else Font.Color := FColorMouseOver;

 end else begin

  Font.Color := FColorNormal;

 end;
end;

procedure TRaylightURLLabel.OnMenuClick(Sender: TObject);
begin
 Clipboard.AsText := Furl;
end;

procedure TRaylightURLLabel.OnMenuPopup(Sender: TObject);
begin
 FMouseDown := false;
 FMouseIn := false;

 adjustLabel;
end;

procedure Register;
begin
  RegisterComponents('Raylight Software', [TRAYLIGHTURLLABEL]);
end;

end.
